@@ -20,6 +20,7 @@
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusPendingCall>
+#include <QDBusPendingReply>
 #include <QDesktopServices>
 #include <QFile>
 #include <QGuiApplication>
@@ -64,6 +65,7 @@
 #include "plasmauserfeedback.h"
 #endif
 #include "PowerManagementInterface.h"
+#include "RefreshNotifier.h"
 #include "discoversettings.h"
 
 using namespace Qt::StringLiterals;
@@ -95,6 +97,8 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     delete factory;
     m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
 
+    new RefreshNotifier(this);
+
     const auto uriApp = "org.kde.discover.app";
 
     qmlRegisterType<UnityLauncher>(uriApp, 1, 0, "UnityLauncher");
@@ -103,8 +107,10 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     qmlRegisterType<OdrsAppsModel>(uriApp, 1, 0, "OdrsAppsModel");
     qmlRegisterType<PowerManagementInterface>(uriApp, 1, 0, "PowerManagementInterface");
 #ifdef WITH_FEEDBACK
-    qmlRegisterSingletonType<PlasmaUserFeedback>(uriApp, 1, 0, "UserFeedbackSettings", [](QQmlEngine *, QJSEngine *) -> QObject * {
-        return new PlasmaUserFeedback(KSharedConfig::openConfig(QStringLiteral("PlasmaUserFeedback"), KConfig::NoGlobals));
+    qmlRegisterSingletonType<PlasmaUserFeedback>(uriApp, 1, 0, "UserFeedbackSettings", [](QQmlEngine *engine, QJSEngine *) -> QObject * {
+        auto r = new PlasmaUserFeedback(KSharedConfig::openConfig(QStringLiteral("PlasmaUserFeedback"), KConfig::NoGlobals));
+        r->setParent(engine);
+        return r;
     });
 #endif
     qmlRegisterSingletonType<DiscoverSettings>(uriApp, 1, 0, "DiscoverSettings", [](QQmlEngine *engine, QJSEngine *) -> QObject * {
@@ -161,14 +167,17 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
     auto action = new OneTimeAction(
         [this]() {
             bool found = DiscoverBackendsFactory::hasRequestedBackends();
+            bool hasPackageKit = false;
             const auto backends = ResourcesModel::global()->backends();
             for (auto b : backends) {
                 found |= b->hasApplications();
+                hasPackageKit = hasPackageKit || (b->name() == "packagekit-backend"_L1 && b->isValid());
             }
 
+            const KOSRelease osRelease;
+            const QString distroName = osRelease.name();
+            const bool isArch = osRelease.id() == u"arch"_s || osRelease.idLike().contains(u"arch"_s);
             if (!found) {
-                const QString distroName = KOSRelease().name();
-
                 QString errorText =
                     i18nc("@title %1 is the distro name", "%1 is not configured for installing apps through Discover—only app add-ons", distroName);
                 QString errorExplanation = xi18nc("@info:usagetip %1 is the distro name",
@@ -176,13 +185,12 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
                                                   "</interface> page, under <interface>Missing Backends</interface>.");
                 QString buttonIcon = QStringLiteral("tools-report-bug");
                 QString buttonText = i18nc("@action:button %1 is the distro name", "Report This Issue to %1", distroName);
-                QString buttonUrl = KOSRelease().bugReportUrl();
+                QString buttonUrl = osRelease.bugReportUrl();
 
-                if (distroName.contains(QStringLiteral("Arch Linux"))) {
+                if (isArch) {
                     errorExplanation = xi18nc("@info:usagetip %1 is the distro name; in this case it always contains 'Arch Linux'",
                                               "To use Discover for apps, install"
-                                              " <link url='https://wiki.archlinux.org/title/Flatpak#Installation'>Flatpak</link> or"
-                                              " <link url='https://wiki.archlinux.org/title/KDE#Discover_does_not_show_any_applications'>PackageKit</link>"
+                                              " <link url='https://wiki.archlinux.org/title/Flatpak#Installation'>Flatpak</link>"
                                               " using the <command>pacman</command> package manager.<nl/><nl/>"
                                               " Review <link url='https://archlinux.org/packages/extra/x86_64/discover/'>%1's packaging for Discover</link>",
                                               distroName);
@@ -192,6 +200,17 @@ DiscoverObject::DiscoverObject(const QVariantMap &initialProperties)
                 }
 
                 Q_EMIT openErrorPage(errorText, errorExplanation, buttonText, buttonIcon, buttonUrl);
+            } else if (hasPackageKit && isArch) {
+                const QString errorText = xi18nc("@info:usagetip %1 is the distro name",
+                                                 "Support for managing packages from %1 is incomplete; you may experience any number of problems."
+                                                 " Do not report bugs to KDE. It is highly recommended to uninstall the"
+                                                 " <resource>packagekit-qt6</resource> package and use Discover only to manage Flatpaks, Snaps,"
+                                                 " and Add-Ons."
+                                                 "<para>%1 maintainers recommended instead using the <command>pacman</command> command-line tool"
+                                                 " to manage packages.</para>",
+                                                 distroName);
+                m_homePageMessage = std::make_unique<InlineMessage>(InlineMessage::Warning, QString(), errorText);
+                Q_EMIT homeMessageChanged();
             }
         },
         this);
@@ -497,13 +516,6 @@ void DiscoverObject::initMainWindow(QQuickWindow *mainWindow)
 
     m_mainWindow = std::unique_ptr<QQuickWindow>(mainWindow);
 
-    KConfigGroup window(KSharedConfig::openConfig(), u"Window"_s);
-    if (window.hasKey("geometry"))
-        m_mainWindow->setGeometry(window.readEntry("geometry", QRect()));
-    if (window.hasKey("visibility")) {
-        QWindow::Visibility visibility(QWindow::Visibility(window.readEntry<int>("visibility", QWindow::Windowed)));
-        m_mainWindow->setVisibility(qMax(visibility, QQuickView::AutomaticVisibility));
-    }
     connect(m_mainWindow.get(), &QQuickView::sceneGraphError, this, [](QQuickWindow::SceneGraphError /*error*/, const QString &message) {
         KCrash::setErrorMessage(message);
         qFatal("%s", qPrintable(message));
@@ -532,10 +544,6 @@ bool DiscoverObject::eventFilter(QObject *object, QEvent *event)
         if (!quitWhenIdle()) {
             return true;
         }
-
-        KConfigGroup window(KSharedConfig::openConfig(), u"Window"_s);
-        window.writeEntry("geometry", m_mainWindow->geometry());
-        window.writeEntry<int>("visibility", m_mainWindow->visibility());
     }
     // } else if (event->type() == QEvent::ShortcutOverride) {
     //     qCWarning(DISCOVER_LOG) << "Action conflict" << event;
@@ -623,17 +631,32 @@ void DiscoverObject::copyTextToClipboard(const QString text)
     clipboard->setText(text);
 }
 
-void DiscoverObject::reboot()
+void DiscoverObject::setAboutToPowerOff()
+{
+    for (auto backend : ResourcesModel::global()->backends()) {
+        backend->aboutTo(AbstractResourcesBackend::PowerOff);
+    }
+}
+
+void DiscoverObject::setAboutToReboot()
+{
+    for (auto backend : ResourcesModel::global()->backends()) {
+        backend->aboutTo(AbstractResourcesBackend::Reboot);
+    }
+}
+
+void DiscoverObject::promptReboot()
 {
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
                                                  QStringLiteral("/LogoutPrompt"),
                                                  QStringLiteral("org.kde.LogoutPrompt"),
-                                                 QStringLiteral("promptReboot"));
+                                                 QStringLiteral("promptAll"));
     QDBusConnection::sessionBus().asyncCall(method);
 }
 
 void DiscoverObject::rebootNow()
 {
+    setAboutToReboot();
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
                                                  QStringLiteral("/org/freedesktop/login1"),
                                                  QStringLiteral("org.freedesktop.login1.Manager"),
@@ -642,10 +665,23 @@ void DiscoverObject::rebootNow()
     QDBusConnection::systemBus().asyncCall(method);
 }
 
-QRect DiscoverObject::initialGeometry() const
+void DiscoverObject::shutdownNow()
 {
-    KConfigGroup window(KSharedConfig::openConfig(), u"Window"_s);
-    return window.readEntry("geometry", QRect());
+    bool shouldRebootFirst = false;
+    for (auto backend : ResourcesModel::global()->backends()) {
+        if (backend->needsRebootForPowerOffAction()) {
+            shouldRebootFirst = true;
+            break;
+        }
+    }
+    setAboutToPowerOff();
+
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
+                                                 QStringLiteral("/org/freedesktop/login1"),
+                                                 QStringLiteral("org.freedesktop.login1.Manager"),
+                                                 shouldRebootFirst ? QStringLiteral("Reboot") : QStringLiteral("PowerOff"));
+    method.setArguments({true /*interactive*/});
+    QDBusConnection::systemBus().asyncCall(method);
 }
 
 QString DiscoverObject::describeSources() const
@@ -653,5 +689,10 @@ QString DiscoverObject::describeSources() const
     return mainWindow()->property("describeSources").toString();
 }
 
+InlineMessage *DiscoverObject::homePageMessage() const
+{
+    return m_homePageMessage.get();
+}
+
 #include "DiscoverObject.moc"
 #include "moc_DiscoverObject.cpp"
