@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef FWUPDRESOURCE_H
-#define FWUPDRESOURCE_H
+#pragma once
 
 #include "FwupdBackend.h"
 
@@ -44,7 +43,7 @@ public:
     QString vendor() const;
     AbstractResource::Type type() const override
     {
-        return Technical;
+        return System;
     }
     bool canExecute() const override
     {
@@ -152,5 +151,3 @@ private:
     bool m_needsReboot = false; // True if device needs Reboot
     QString m_origin;
 };
-
-#endif // FWUPDRESOURCE_H
