@@ -94,9 +94,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QString upgradeText READ upgradeText NOTIFY versionsChanged)
     Q_PROPERTY(bool isRemovable READ isRemovable CONSTANT)
     Q_PROPERTY(QString versionString READ versionString NOTIFY versionsChanged)
-    Q_PROPERTY(QString contentRatingText READ contentRatingText CONSTANT)
     Q_PROPERTY(QString contentRatingDescription READ contentRatingDescription CONSTANT)
-    Q_PROPERTY(ContentIntensity contentRatingIntensity READ contentRatingIntensity CONSTANT)
     Q_PROPERTY(uint contentRatingMinimumAge READ contentRatingMinimumAge CONSTANT)
     Q_PROPERTY(QStringList topObjects READ topObjects CONSTANT)
     Q_PROPERTY(QStringList bottomObjects READ bottomObjects CONSTANT)
@@ -128,12 +126,6 @@ public:
     };
     Q_ENUM(State)
 
-    enum ContentIntensity {
-        Mild,
-        Intense,
-    };
-    Q_ENUM(ContentIntensity)
-
     /**
      * Constructs the AbstractResource with its corresponding backend
      */
@@ -176,7 +168,14 @@ public:
     enum Type {
         Application,
         Addon,
-        Technical,
+        /**
+         * This is where Flatpak runtimes would go
+         */
+        ApplicationSupport,
+        /**
+         * This is where system updates would go
+         */
+        System,
     };
     Q_ENUM(Type)
     virtual Type type() const = 0;
@@ -205,8 +204,8 @@ public:
 
     virtual QString appstreamId() const;
 
-    void addMetadata(const QString &key, const QJsonValue &value);
-    QJsonValue getMetadata(const QString &key);
+    void addMetadata(const QLatin1StringView &key, const QJsonValue &value);
+    QJsonValue getMetadata(const QLatin1StringView &key);
 
     bool canUpgrade();
     bool isInstalled();
@@ -262,8 +261,6 @@ public:
 
     virtual QString versionString();
 
-    virtual QString contentRatingText() const;
-    virtual ContentIntensity contentRatingIntensity() const;
     virtual QString contentRatingDescription() const;
     virtual uint contentRatingMinimumAge() const;
 
@@ -275,6 +272,8 @@ public:
      * @returns List of component URLs to display at the bottom of application page.
      */
     virtual QStringList bottomObjects() const;
+    [[nodiscard]] virtual bool hasResolvedIcon() const;
+    virtual void resolveIcon();
 
 public Q_SLOTS:
     virtual void fetchScreenshots();
