@@ -7,7 +7,6 @@
 #include "NotifierItem.h"
 #include "updatessettings.h"
 #include <KLocalizedString>
-#include <QDebug>
 #include <QMenu>
 
 NotifierItem::NotifierItem()
@@ -24,7 +23,7 @@ void NotifierItem::setupNotifierItem()
 
     connect(m_item, &KStatusNotifierItem::activateRequested, &m_notifier, [this]() {
         if (m_notifier.needsReboot()) {
-            m_notifier.reboot();
+            m_notifier.promptAll();
         } else {
             m_notifier.showDiscoverUpdates(m_item->providedToken());
         }
@@ -43,12 +42,22 @@ void NotifierItem::setupNotifierItem()
     });
 
     auto refreshAction = menu->addAction(QIcon::fromTheme(QStringLiteral("view-refresh")), i18n("Refresh…"));
-    connect(refreshAction, &QAction::triggered, &m_notifier, &DiscoverNotifier::recheckSystemUpdateNeeded);
+    connect(refreshAction, &QAction::triggered, &m_notifier, &DiscoverNotifier::recheckSystemUpdateNeededAndNotifyApp);
+
+    if (m_notifier.needsReboot()) {
+        menu->addSeparator();
+
+        auto rebootAction = menu->addAction(QIcon::fromTheme(QStringLiteral("system-reboot-update")), i18n("Install Updates and Restart…"));
+        connect(rebootAction, &QAction::triggered, &m_notifier, &DiscoverNotifier::rebootPrompt);
+
+        auto shutdownAction = menu->addAction(QIcon::fromTheme(QStringLiteral("system-shutdown-update")), i18n("Install Updates and Shut Down…"));
+        connect(shutdownAction, &QAction::triggered, &m_notifier, &DiscoverNotifier::shutdownPrompt);
+    }
 
     auto f = [this]() {
         m_item->setTitle(i18n("Restart to apply installed updates"));
         m_item->setToolTipTitle(i18n("Click to restart the device"));
-        m_item->setIconByName(QStringLiteral("view-refresh"));
+        m_item->setIconByName(QStringLiteral("system-reboot-update"));
     };
     if (m_notifier.needsReboot())
         f();
@@ -114,8 +123,17 @@ bool NotifierItem::shouldShowStatusNotifier() const
     case DiscoverNotifier::Busy:
     case DiscoverNotifier::RebootRequired:
         return true;
-    case DiscoverNotifier::NormalUpdates:
+    case DiscoverNotifier::NormalUpdates: {
+        // Only show the status notifier on next notification time
+        // BUG: 466693
+        const QDateTime earliestNextNotificationTime =
+            m_notifier.settings()->lastNotificationTime().addSecs(m_notifier.settings()->requiredNotificationInterval());
+
+        return m_notifier.settings()->requiredNotificationInterval() > 0 && earliestNextNotificationTime.isValid()
+            && earliestNextNotificationTime < QDateTime::currentDateTimeUtc();
+    }
     case DiscoverNotifier::SecurityUpdates:
+        //...unless it's a security update, which should always be shown if the user wants notifications at all
         return m_notifier.settings()->requiredNotificationInterval() > 0;
     case DiscoverNotifier::Offline:
     case DiscoverNotifier::NoUpdates:
